#!/usr/bin/env perl

# NAME:
# 	chkdiff - regression test for updrev
#
# SYNOPSIS:
#	updrev.pl -D nd_data
#	cvs diff > diffs
# 	chkdiff.pl nd_data diffs > diffs.checked
#	grep PTR diffs.checked | grep -v 'ok$'
#
# DESCRIPTION:
#	This is a tool for verifying that changes made by updrev* are
#	sane.  That is, for regression testing updrev.  The final
#	command in the sequence shown above, should produce no output.
#
#	For each PTR change we check that there is a matching A record
#	for that name and IP address in the data that updrev ran from.
#
#	If no such A record exists then the PTR record should have
#	been dropped.
#
#	If the A record does exist, then the only correct change is
#	for the PTR to have been added - since the goal of updrev is
#	to not alter existing (valid) PTR records.
#
#	The output of this script is the same as the input, except
#	that the PTR changes are tagged with this scripts assessment:
#
#	'ok'	We concurr.
#
#	'should be <'
#		We could not find an A record, so expected the PTR to
#		be dropped.
#
#	'check'
#		We could find an A record, but the PTR was dropped.
#
# BUGS:
#	We invoke grep(1) for every PTR change so can take ages to
#	check changes to a large DNS.
#	
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: chkdiff.pl,v 1.4 2001/08/30 07:48:34 sjg Exp $
#
#	@(#)Copyright (c) 1997-2001 Simon J. Gerraty.
#	All rights reserved.
#	
#	This software is the property of Simon J. Gerraty.
#	It may be used only in accordance with the conditions outlined
#	in the accompanying LICENSE file. If your distribution does
#	not contain a LICENSE file, then you have no rights to use
#	this software and should contact sales@crufty.net to obtain a
#	licensed distribution.
#	
#	This software is provided "AS IS" in the hope that it will be
#	of use.  Since it is not possible to anticipate all possible
#	use of this software, there is absolutely NO WARRANTY.
#
#	Please forward bug reports and fixes to help@crufty.net
#


$data = shift(@ARGV);

if (open(D, "grep '^A' $data |")) {
  while (<D>) {
    chop;
    ($type,$fqdn,$addr) = split(/\s+/);

    $fqdn =~ s/\./\\./g;
    if ($A{$addr} eq '') {
      $A{$addr} = $fqdn;
    } else {
      $A{$addr} .= "|$fqdn";
    }
  }
  close D;
}


while (<>) {
  chop;

  print;
  if (m/^diff.*\s([\d\.]+).rev/) {
    $net = $1;
  } elsif (m/^([<>])\s+(\S+)\s.*PTR\s+(.+)\.$/) {
    $change = $1;
    $ptr = $2;
    $name = $3;

    if ($ptr =~ m/^\s*;/) {		# a comment cannot be wrong :-)
      $ok = 'ok';
    } else {
      @p = split(/\./, $ptr);
      $a = $net;
      for ($i = $#p; $i >= 0; $i--) {
	$a .= ".$p[$i]";
      }
      $ok = $re = $A{$a};
      if ($re ne '' && $name !~ m/^($re)\.*$/) {
	$ok = '';
      }
      if ($ok eq '') {
	# no A record, so '<' is expected
	$ok = ($change eq '<') ? 'ok' : 'should be <';
      } else {
	# an A record exists, so any change should only be '>'
	$ok = ($change eq '>') ? 'ok' : 'check';
      }
    }

    print " $ok";
  }
  print "\n";
}
